@@ -8,6 +8,7 @@
 */
 
 #include "kglobalshortcuttest.h"
+#include "sequencehelpers_p.h"
 #include <QAction>
 #include <QDBusInterface>
 #include <QSignalSpy>
@@ -21,7 +22,11 @@
 #include <QDBusConnectionInterface>
 
 #ifdef HAVE_XCB_XTEST
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
 #include <QX11Info>
+#endif
 #define XK_MISCELLANY
 #define XK_XKB_KEYS
 #include <X11/keysymdef.h>
@@ -196,7 +201,12 @@ void KGlobalShortcutTest::testFindActionByKey()
         QSKIP("kglobalaccel not installed");
     }
 
+#if KGLOBALACCEL_ENABLE_DEPRECATED_SINCE(5, 90)
     QList<KGlobalShortcutInfo> actionId = KGlobalAccel::self()->getGlobalShortcutsByKey(sequenceB);
+#else
+    QList<KGlobalShortcutInfo> actionId = KGlobalAccel::self()->globalShortcutsByKey(sequenceB);
+#endif
+
     QCOMPARE(actionId.size(), 1);
 
     QString actionIdAComponentUniqueName("qttest");
@@ -209,7 +219,12 @@ void KGlobalShortcutTest::testFindActionByKey()
     QCOMPARE(actionId.first().componentFriendlyName(), actionIdAComponentFriendlyName);
     QCOMPARE(actionId.first().friendlyName(), actionIdAFriendlyName);
 
+#if KGLOBALACCEL_ENABLE_DEPRECATED_SINCE(5, 90)
     actionId = KGlobalAccel::self()->getGlobalShortcutsByKey(sequenceA);
+#else
+    actionId = KGlobalAccel::self()->globalShortcutsByKey(sequenceA);
+#endif
+
     QCOMPARE(actionId.size(), 1);
 
     QCOMPARE(actionId.first().componentUniqueName(), actionIdAComponentUniqueName);
@@ -253,6 +268,43 @@ void KGlobalShortcutTest::testChangeShortcut()
     QCOMPARE(KGlobalAccel::self()->shortcut(m_actionB), cutB);
     // Check that the default shortcut is not active
     QVERIFY(KGlobalAccel::self()->defaultShortcut(m_actionB).isEmpty()); // unchanged
+
+    struct SeqInfo {
+        QKeySequence seqA;
+        QKeySequence seqB;
+        bool expected;
+    };
+
+    static const SeqInfo values[] = {
+        {QKeySequence('A', 'B', 'C'), QKeySequence('A', 'B', 'C'), true},
+        {QKeySequence('A', 'B', 'C'), QKeySequence('B', 'C'), true},
+        {QKeySequence('A', 'B', 'C'), QKeySequence('A', 'B'), true},
+        {QKeySequence('A', 'B', 'C'), QKeySequence('B'), true},
+        {QKeySequence('A', 'B', 'C'), QKeySequence('D', 'B', 'C'), false},
+        {QKeySequence('A', 'B', 'C'), QKeySequence('A', 'B', 'D'), false},
+        {QKeySequence('A', 'B', 'C'), QKeySequence('D'), false},
+        {QKeySequence('A', 'B', 'B', 'C'), QKeySequence('B', 'B'), true},
+        {QKeySequence('A', 'B', 'C', 'D'), QKeySequence('A', 'B'), true},
+        {QKeySequence('A', 'B', 'C', 'D'), QKeySequence('B', 'C'), true},
+        {QKeySequence('A', 'B', 'C', 'D'), QKeySequence('C', 'D'), true},
+    };
+
+    for (const auto &info : values) {
+        KGlobalAccel::self()->removeAllShortcuts(m_actionA);
+        KGlobalAccel::self()->setGlobalShortcut(m_actionA, info.seqA);
+
+        cutB.clear();
+        cutB << info.seqB;
+        KGlobalAccel::self()->removeAllShortcuts(m_actionB);
+        KGlobalAccel::self()->setGlobalShortcut(m_actionB, info.seqB);
+
+        QList<QKeySequence> seq = KGlobalAccel::self()->shortcut(m_actionB);
+        if (!info.expected) {
+            QCOMPARE(seq, cutB);
+        } else {
+            QVERIFY(seq.first().isEmpty());
+        }
+    }
 }
 
 void KGlobalShortcutTest::testStealShortcut()
@@ -481,6 +533,93 @@ void KGlobalShortcutTest::testGetGlobalShortcut()
     //     QCOMPARE(shortcutList.count(), 1);
 }
 
+void KGlobalShortcutTest::testMangle()
+{
+    setupTest("testMangle");
+    static const std::pair<QKeySequence, QKeySequence> values[] = {
+        {QKeySequence('A', 'B', 'C', 'D'), QKeySequence('A', 'B', 'C', 'D')},
+        {QKeySequence("Shift+A,B,C,D"), QKeySequence("Shift+A,B,C,D")},
+        {QKeySequence("Shift+Tab,B,C,D"), QKeySequence("Shift+Tab,B,C,D")},
+        {QKeySequence("Ctrl+Shift+Tab,B,C,D"), QKeySequence("Ctrl+Shift+Tab,B,C,D")},
+        {QKeySequence("Shift+BackTab,B,C,D"), QKeySequence("Shift+Tab,B,C,D")},
+        {QKeySequence("Shift+BackTab,Shift+BackTab,Shift+BackTab,Shift+BackTab"), QKeySequence("Shift+Tab,Shift+Tab,Shift+Tab,Shift+Tab")},
+        {QKeySequence(), QKeySequence()},
+    };
+
+    for (const auto &pair : values) {
+        QCOMPARE(Utils::mangleKey(pair.first), pair.second);
+    }
+}
+
+void KGlobalShortcutTest::testCrop()
+{
+    setupTest("testCrop");
+
+    struct SeqInfo {
+        QKeySequence seq;
+        int count = 0;
+        QKeySequence expected;
+    };
+
+    static const SeqInfo values[] = {
+        {QKeySequence('A', 'B', 'C', 'D'), 1, QKeySequence('B', 'C', 'D')},
+        {QKeySequence('A', 'B', 'C', 'D'), 2, QKeySequence('C', 'D')},
+        {QKeySequence('A', 'B', 'C'), 1, QKeySequence('B', 'C')},
+        {QKeySequence('A', 'B', 'C'), 2, QKeySequence('C')},
+        {QKeySequence('A'), 1, QKeySequence()},
+        {QKeySequence('A'), 2, QKeySequence()},
+        {QKeySequence(), 1, QKeySequence()},
+        {QKeySequence(), 2, QKeySequence()},
+    };
+
+    for (const auto &s : values) {
+        QCOMPARE(Utils::cropKey(s.seq, s.count), s.expected);
+    }
+}
+
+void KGlobalShortcutTest::testReverse()
+{
+    setupTest("testReverse");
+    static const std::pair<QKeySequence, QKeySequence> values[] = {
+        {QKeySequence('A', 'B', 'C', 'D'), QKeySequence('D', 'C', 'B', 'A')},
+        {QKeySequence('A', 'B', 'C'), QKeySequence('C', 'B', 'A')},
+        {QKeySequence('A', 'B'), QKeySequence('B', 'A')},
+        {QKeySequence('A'), QKeySequence('A')},
+        {QKeySequence(), QKeySequence()},
+    };
+
+    for (const auto &pair : values) {
+        QCOMPARE(Utils::reverseKey(pair.first), pair.second);
+    }
+}
+
+void KGlobalShortcutTest::testMatch()
+{
+    setupTest("Match");
+
+    struct Info {
+        QList<QKeySequence> seqList;
+        bool isMatch = false;
+    };
+
+    static const Info values[] = {
+        {QList{QKeySequence('A', 'B', 'C')}, true},
+        {QList{QKeySequence('A', 'B', 'D')}, false},
+        {QList{QKeySequence('D')}, false},
+        {QList{QKeySequence('A', 'B', 'C', 'D')}, true},
+        {QList{QKeySequence('D', 'A', 'B', 'C')}, true},
+        {QList{QKeySequence('D', 'A', 'B', 'D')}, false},
+        {QList{QKeySequence('A', 'B')}, true},
+        {QList{QKeySequence('B', 'C')}, true},
+        {QList{QKeySequence('A', 'C')}, false},
+    };
+
+    const QKeySequence seq = QKeySequence('A', 'B', 'C');
+    for (const auto &i : values) {
+        QCOMPARE(Utils::matchSequences(seq, i.seqList), i.isMatch);
+    }
+}
+
 void KGlobalShortcutTest::testForgetGlobalShortcut()
 {
     setupTest("testForgetGlobalShortcut");
@@ -509,3 +648,5 @@ void KGlobalShortcutTest::testForgetGlobalShortcut()
     QVERIFY(!components.contains(componentId));
 #endif
 }
+
+#include "moc_kglobalshortcuttest.cpp"
