@@ -9,21 +9,26 @@
 */
 
 #include "kglobalaccel.h"
-#include "kglobalaccel_p.h"
 #include "kglobalaccel_debug.h"
+#include "kglobalaccel_p.h"
 
 #include <memory>
 
 #include <QAction>
+#include <QDBusMessage>
+#include <QDBusMetaType>
 #include <QGuiApplication>
 #include <QMessageBox>
 #include <QPushButton>
-#include <QDBusMessage>
-#include <QDBusMetaType>
 #include <config-kglobalaccel.h>
+
 #if HAVE_X11
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
 #include <QX11Info>
 #endif
+#endif
 
 org::kde::kglobalaccel::Component *KGlobalAccelPrivate::getComponent(const QString &componentUnique, bool remember = false)
 {
@@ -35,21 +40,10 @@ org::kde::kglobalaccel::Component *KGlobalAccelPrivate::getComponent(const QStri
         }
     }
 
-    // Connect to the kglobalaccel daemon
-    org::kde::KGlobalAccel kglobalaccel(
-        QStringLiteral("org.kde.kglobalaccel"),
-        QStringLiteral("/kglobalaccel"),
-        QDBusConnection::sessionBus());
-    if (!kglobalaccel.isValid()) {
-        qCDebug(KGLOBALACCEL_LOG) << "Failed to connect to the kglobalaccel daemon" << QDBusConnection::sessionBus().lastError();
-        return nullptr;
-    }
-
     // Get the path for our component. We have to do that because
     // componentUnique is probably not a valid dbus object path
-    QDBusReply<QDBusObjectPath> reply = kglobalaccel.getComponent(componentUnique);
+    QDBusReply<QDBusObjectPath> reply = iface()->getComponent(componentUnique);
     if (!reply.isValid()) {
-
         if (reply.error().name() == QLatin1String("org.kde.kglobalaccel.NoSuchComponent")) {
             // No problem. The component doesn't exists. That's normal
             return nullptr;
@@ -61,11 +55,8 @@ org::kde::kglobalaccel::Component *KGlobalAccelPrivate::getComponent(const QStri
     }
 
     // Now get the component
-    org::kde::kglobalaccel::Component *component = new org::kde::kglobalaccel::Component(
-        QStringLiteral("org.kde.kglobalaccel"),
-        reply.value().path(),
-        QDBusConnection::sessionBus(),
-        q);
+    org::kde::kglobalaccel::Component *component =
+        new org::kde::kglobalaccel::Component(QStringLiteral("org.kde.kglobalaccel"), reply.value().path(), QDBusConnection::sessionBus(), q);
 
     // No component no cleaning
     if (!component->isValid()) {
@@ -75,8 +66,19 @@ org::kde::kglobalaccel::Component *KGlobalAccelPrivate::getComponent(const QStri
 
     if (remember) {
         // Connect to the signals we are interested in.
-        q->connect(component, SIGNAL(globalShortcutPressed(QString,QString,qlonglong)),
-                   SLOT(_k_invokeAction(QString,QString,qlonglong)));
+        q->connect(component,
+                   &org::kde::kglobalaccel::Component::globalShortcutPressed,
+                   q,
+                   [this](const QString &componentUnique, const QString &shortcutUnique, qlonglong timestamp) {
+                       invokeAction(componentUnique, shortcutUnique, timestamp);
+                   });
+
+        q->connect(component,
+                   &org::kde::kglobalaccel::Component::globalShortcutReleased,
+                   q,
+                   [this](const QString &componentUnique, const QString &shortcutUnique, qlonglong) {
+                       invokeDeactivate(componentUnique, shortcutUnique);
+                   });
 
         components[componentUnique] = component;
     }
@@ -101,20 +103,13 @@ void KGlobalAccelPrivate::cleanup()
     m_watcher = nullptr;
 }
 
-KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
-    :
-#if KGLOBALACCEL_BUILD_DEPRECATED_SINCE(4, 4)
-    enabled(true),
-#endif
-    q(q),
-    m_iface(nullptr)
+KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *qq)
+    : q(qq)
 {
-    m_watcher = new QDBusServiceWatcher(serviceName(),
-            QDBusConnection::sessionBus(),
-            QDBusServiceWatcher::WatchForOwnerChange,
-            q);
-    q->connect(m_watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-               q, SLOT(_k_serviceOwnerChanged(QString,QString,QString)));
+    m_watcher = new QDBusServiceWatcher(serviceName(), QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, q);
+    q->connect(m_watcher, &QDBusServiceWatcher::serviceOwnerChanged, q, [this](const QString &serviceName, const QString &oldOwner, const QString &newOwner) {
+        serviceOwnerChanged(serviceName, oldOwner, newOwner);
+    });
 }
 
 org::kde::KGlobalAccel *KGlobalAccelPrivate::iface()
@@ -129,8 +124,16 @@ org::kde::KGlobalAccel *KGlobalAccelPrivate::iface()
                 qCritical() << "Couldn't start kglobalaccel from org.kde.kglobalaccel.service:" << reply.error();
             }
         }
-        q->connect(m_iface, SIGNAL(yourShortcutGotChanged(QStringList,QList<int>)),
-                SLOT(_k_shortcutGotChanged(QStringList,QList<int>)));
+
+#if KGLOBALACCEL_BUILD_DEPRECATED_SINCE(5, 90)
+        q->connect(m_iface, &org::kde::KGlobalAccel::yourShortcutGotChanged, q, [this](const QStringList &actionId, const QList<int> &newKeys) {
+            shortcutGotChanged(actionId, newKeys);
+        });
+#endif
+
+        q->connect(m_iface, &org::kde::KGlobalAccel::yourShortcutsChanged, q, [this](const QStringList &actionId, const QList<QKeySequence> &newKeys) {
+            shortcutsChanged(actionId, newKeys);
+        });
     }
     return m_iface;
 }
@@ -138,10 +141,13 @@ org::kde::KGlobalAccel *KGlobalAccelPrivate::iface()
 KGlobalAccel::KGlobalAccel()
     : d(new KGlobalAccelPrivate(this))
 {
-    qDBusRegisterMetaType<QList<int> >();
-    qDBusRegisterMetaType<QList<QStringList> >();
+    qDBusRegisterMetaType<QList<int>>();
+    qDBusRegisterMetaType<QKeySequence>();
+    qDBusRegisterMetaType<QList<QKeySequence>>();
+    qDBusRegisterMetaType<QList<QStringList>>();
     qDBusRegisterMetaType<KGlobalShortcutInfo>();
-    qDBusRegisterMetaType<QList<KGlobalShortcutInfo> >();
+    qDBusRegisterMetaType<QList<KGlobalShortcutInfo>>();
+    qDBusRegisterMetaType<KGlobalAccel::MatchType>();
 }
 
 KGlobalAccel::~KGlobalAccel()
@@ -149,15 +155,14 @@ KGlobalAccel::~KGlobalAccel()
     delete d;
 }
 
-void KGlobalAccel::activateGlobalShortcutContext(
-    const QString &contextUnique,
-    const QString &contextFriendly,
-    const QString &programName)
+#if KGLOBALACCEL_BUILD_DEPRECATED_SINCE(5, 102)
+void KGlobalAccel::activateGlobalShortcutContext(const QString &contextUnique, const QString &contextFriendly, const QString &programName)
 {
     Q_UNUSED(contextFriendly);
     // TODO: provide contextFriendly
     self()->d->iface()->activateGlobalShortcutContext(programName, contextUnique);
 }
+#endif
 
 // static
 bool KGlobalAccel::cleanComponent(const QString &componentUnique)
@@ -226,7 +231,7 @@ bool KGlobalAccelPrivate::doRegister(QAction *action)
 {
     if (!action || action->objectName().isEmpty() || action->objectName().startsWith(QLatin1String("unnamed-"))) {
         qWarning() << "Attempt to set global shortcut for action without objectName()."
-                   " Read the setGlobalShortcut() documentation.";
+                      " Read the setGlobalShortcut() documentation.";
         return false;
     }
 
@@ -252,7 +257,7 @@ bool KGlobalAccelPrivate::doRegister(QAction *action)
 
 void KGlobalAccelPrivate::remove(QAction *action, Removal removal)
 {
-    if (!action  || action->objectName().isEmpty()) {
+    if (!action || action->objectName().isEmpty()) {
         return;
     }
 
@@ -280,9 +285,11 @@ void KGlobalAccelPrivate::remove(QAction *action, Removal removal)
 
         if (!action->property("isConfigurationAction").toBool()) {
             // If it's a session shortcut unregister it.
-            action->objectName().startsWith(QLatin1String("_k_session:"))
-            ? unregister(actionId)
-            : setInactive(actionId);
+            if (action->objectName().startsWith(QLatin1String("_k_session:"))) {
+                unregister(actionId);
+            } else {
+                setInactive(actionId);
+            }
         }
     }
 
@@ -292,10 +299,13 @@ void KGlobalAccelPrivate::remove(QAction *action, Removal removal)
 
 void KGlobalAccelPrivate::unregister(const QStringList &actionId)
 {
-    auto message = QDBusMessage::createMethodCall(iface()->service(), iface()->path(), iface()->interface(), QStringLiteral("unRegister"));
-    message.setArguments({actionId});
+    const auto component = actionId.at(KGlobalAccel::ComponentUnique);
+    const auto action = actionId.at(KGlobalAccel::ActionUnique);
+
+    auto message = QDBusMessage::createMethodCall(iface()->service(), iface()->path(), iface()->interface(), QStringLiteral("unregister"));
+    message.setArguments({component, action});
     message.setAutoStartService(false);
-    QDBusConnection::sessionBus().call(message, QDBus::NoBlock);
+    QDBusConnection::sessionBus().call(message);
 }
 
 void KGlobalAccelPrivate::setInactive(const QStringList &actionId)
@@ -303,10 +313,10 @@ void KGlobalAccelPrivate::setInactive(const QStringList &actionId)
     auto message = QDBusMessage::createMethodCall(iface()->service(), iface()->path(), iface()->interface(), QStringLiteral("setInactive"));
     message.setArguments({actionId});
     message.setAutoStartService(false);
-    QDBusConnection::sessionBus().call(message, QDBus::NoBlock);
+    QDBusConnection::sessionBus().call(message);
 }
 
-void KGlobalAccelPrivate::updateGlobalShortcut(/*const would be better*/QAction* action,
+void KGlobalAccelPrivate::updateGlobalShortcut(/*const would be better*/ QAction *action,
                                                ShortcutTypes actionFlags,
                                                KGlobalAccel::GlobalShortcutLoading globalFlags)
 {
@@ -333,16 +343,13 @@ void KGlobalAccelPrivate::updateGlobalShortcut(/*const would be better*/QAction*
         }
 
         // Sets the shortcut, returns the active/real keys
-        const auto result = iface()->setShortcut(
-                                      actionId,
-                                      intListFromShortcut(activeShortcut),
-                                      activeSetterFlags);
+        const auto result = iface()->setShortcutKeys(actionId, activeShortcut, activeSetterFlags);
 
         // Make sure we get informed about changes in the component by kglobalaccel
         getComponent(componentUniqueForAction(action), true);
 
         // Create a shortcut from the result
-        const QList<QKeySequence> scResult(shortcutFromIntList(result));
+        const QList<QKeySequence> scResult(result);
 
         if (isConfigurationAction && (globalFlags & NoAutoloading)) {
             // If this is a configuration action and we have set the shortcut,
@@ -352,35 +359,34 @@ void KGlobalAccelPrivate::updateGlobalShortcut(/*const would be better*/QAction*
             // at the time of comparison (now) the action *already has* the new shortcut.
             // We called setShortcut(), remember?
             // Also note that we will see our own signal so we may not need to call
-            // setActiveGlobalShortcutNoEnable - _k_shortcutGotChanged() does it.
+            // setActiveGlobalShortcutNoEnable - shortcutGotChanged() does it.
             // In practice it's probably better to get the change propagated here without
             // DBus delay as we do below.
-            iface()->setForeignShortcut(actionId, result);
+            iface()->setForeignShortcutKeys(actionId, result);
         }
         if (scResult != activeShortcut) {
             // If kglobalaccel returned a shortcut that differs from the one we
             // sent, use that one. There must have been clashes or some other problem.
             actionShortcuts.insert(action, scResult);
-            emit q->globalShortcutChanged(action, scResult.isEmpty() ? QKeySequence() : scResult.first());
+            Q_EMIT q->globalShortcutChanged(action, scResult.isEmpty() ? QKeySequence() : scResult.first());
         }
     }
 
     if (actionFlags & DefaultShortcut) {
         const QList<QKeySequence> defaultShortcut = actionDefaultShortcuts.value(action);
-        iface()->setShortcut(actionId, intListFromShortcut(defaultShortcut),
-                          setterFlags | IsDefault);
+        iface()->setShortcutKeys(actionId, defaultShortcut, setterFlags | IsDefault);
     }
 }
 
 QStringList KGlobalAccelPrivate::makeActionId(const QAction *action)
 {
-    QStringList ret(componentUniqueForAction(action));  // Component Unique Id ( see actionIdFields )
+    QStringList ret(componentUniqueForAction(action)); // Component Unique Id ( see actionIdFields )
     Q_ASSERT(!ret.at(KGlobalAccel::ComponentUnique).isEmpty());
     Q_ASSERT(!action->objectName().isEmpty());
-    ret.append(action->objectName());                   // Action Unique Name
-    ret.append(componentFriendlyForAction(action));     // Component Friendly name
+    ret.append(action->objectName()); // Action Unique Name
+    ret.append(componentFriendlyForAction(action)); // Component Friendly name
     const QString actionText = action->text().replace(QLatin1Char('&'), QStringLiteral(""));
-    ret.append(actionText);                             // Action Friendly Name
+    ret.append(actionText); // Action Friendly Name
     return ret;
 }
 
@@ -388,7 +394,11 @@ QList<int> KGlobalAccelPrivate::intListFromShortcut(const QList<QKeySequence> &c
 {
     QList<int> ret;
     for (const QKeySequence &sequence : cut) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        ret.append(sequence[0].toCombined());
+#else
         ret.append(sequence[0]);
+#endif
     }
     while (!ret.isEmpty() && ret.last() == 0) {
         ret.removeLast();
@@ -399,9 +409,10 @@ QList<int> KGlobalAccelPrivate::intListFromShortcut(const QList<QKeySequence> &c
 QList<QKeySequence> KGlobalAccelPrivate::shortcutFromIntList(const QList<int> &list)
 {
     QList<QKeySequence> ret;
-    for (int i : list) {
-        ret.append(i);
-    }
+    ret.reserve(list.size());
+    std::transform(list.begin(), list.end(), std::back_inserter(ret), [](int i) {
+        return QKeySequence(i);
+    });
     return ret;
 }
 
@@ -427,21 +438,18 @@ QString KGlobalAccelPrivate::componentFriendlyForAction(const QAction *action)
 }
 
 #if HAVE_X11
-int _k_timestampCompare(unsigned long time1_, unsigned long time2_) // like strcmp()
+int timestampCompare(unsigned long time1_, unsigned long time2_) // like strcmp()
 {
     quint32 time1 = time1_;
     quint32 time2 = time2_;
     if (time1 == time2) {
         return 0;
     }
-    return quint32(time1 - time2) < 0x7fffffffU ? 1 : -1;   // time1 > time2 -> 1, handle wrapping
+    return quint32(time1 - time2) < 0x7fffffffU ? 1 : -1; // time1 > time2 -> 1, handle wrapping
 }
 #endif
 
-void KGlobalAccelPrivate::_k_invokeAction(
-    const QString &componentUnique,
-    const QString &actionUnique,
-    qlonglong timestamp)
+QAction *KGlobalAccelPrivate::findAction(const QString &componentUnique, const QString &actionUnique)
 {
     QAction *action = nullptr;
     const QList<QAction *> candidates = nameToAction.values(actionUnique);
@@ -456,6 +464,15 @@ void KGlobalAccelPrivate::_k_invokeAction(
     // - the action is not enabled
     // - the action is an configuration action
     if (!action || !action->isEnabled() || action->property("isConfigurationAction").toBool()) {
+        return nullptr;
+    }
+    return action;
+}
+
+void KGlobalAccelPrivate::invokeAction(const QString &componentUnique, const QString &actionUnique, qlonglong timestamp)
+{
+    QAction *action = findAction(componentUnique, actionUnique);
+    if (!action) {
         return;
     }
 
@@ -465,21 +482,37 @@ void KGlobalAccelPrivate::_k_invokeAction(
     // in the proper place in relation to the X events queue in order to avoid
     // the possibility of wrong ordering of user events.
     if (QX11Info::isPlatformX11()) {
-        if (_k_timestampCompare(timestamp, QX11Info::appTime()) > 0) {
+        if (timestampCompare(timestamp, QX11Info::appTime()) > 0) {
             QX11Info::setAppTime(timestamp);
         }
-        if (_k_timestampCompare(timestamp, QX11Info::appUserTime()) > 0) {
+        if (timestampCompare(timestamp, QX11Info::appUserTime()) > 0) {
             QX11Info::setAppUserTime(timestamp);
         }
     }
 #endif
     action->setProperty("org.kde.kglobalaccel.activationTimestamp", timestamp);
 
+    if (m_lastActivatedAction != action) {
+        Q_EMIT q->globalShortcutActiveChanged(action, true);
+        m_lastActivatedAction = action;
+    }
     action->trigger();
 }
 
-void KGlobalAccelPrivate::_k_shortcutGotChanged(const QStringList &actionId,
-        const QList<int> &keys)
+void KGlobalAccelPrivate::invokeDeactivate(const QString &componentUnique, const QString &actionUnique)
+{
+    QAction *action = findAction(componentUnique, actionUnique);
+    if (!action) {
+        return;
+    }
+
+    m_lastActivatedAction.clear();
+
+    Q_EMIT q->globalShortcutActiveChanged(action, false);
+}
+
+#if KGLOBALACCEL_BUILD_DEPRECATED_SINCE(5, 90)
+void KGlobalAccelPrivate::shortcutGotChanged(const QStringList &actionId, const QList<int> &keys)
 {
     QAction *action = nameToAction.value(actionId.at(KGlobalAccel::ActionUnique));
     if (!action) {
@@ -488,11 +521,22 @@ void KGlobalAccelPrivate::_k_shortcutGotChanged(const QStringList &actionId,
 
     const QList<QKeySequence> shortcuts = shortcutFromIntList(keys);
     actionShortcuts.insert(action, shortcuts);
-    emit q->globalShortcutChanged(action, keys.isEmpty() ? QKeySequence() : shortcuts.first());
+    Q_EMIT q->globalShortcutChanged(action, keys.isEmpty() ? QKeySequence() : shortcuts.first());
+}
+#endif
+
+void KGlobalAccelPrivate::shortcutsChanged(const QStringList &actionId, const QList<QKeySequence> &keys)
+{
+    QAction *action = nameToAction.value(actionId.at(KGlobalAccel::ActionUnique));
+    if (!action) {
+        return;
+    }
+
+    actionShortcuts.insert(action, keys);
+    Q_EMIT q->globalShortcutChanged(action, keys.isEmpty() ? QKeySequence() : keys.first());
 }
 
-void KGlobalAccelPrivate::_k_serviceOwnerChanged(const QString &name, const QString &oldOwner,
-        const QString &newOwner)
+void KGlobalAccelPrivate::serviceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner)
 {
     Q_UNUSED(oldOwner);
     if (name == QLatin1String("org.kde.kglobalaccel") && !newOwner.isEmpty()) {
@@ -504,13 +548,13 @@ void KGlobalAccelPrivate::_k_serviceOwnerChanged(const QString &name, const QStr
 
 void KGlobalAccelPrivate::reRegisterAll()
 {
-    //We clear all our data, assume that all data on the other side is clear too,
-    //and register each action as if it just was allowed to have global shortcuts.
-    //If the kded side still has the data it doesn't matter because of the
-    //autoloading mechanism. The worst case I can imagine is that an action's
-    //shortcut was changed but the kded side died before it got the message so
-    //autoloading will now assign an old shortcut to the action. Particularly
-    //picky apps might assert or misbehave.
+    // We clear all our data, assume that all data on the other side is clear too,
+    // and register each action as if it just was allowed to have global shortcuts.
+    // If the kded side still has the data it doesn't matter because of the
+    // autoloading mechanism. The worst case I can imagine is that an action's
+    // shortcut was changed but the kded side died before it got the message so
+    // autoloading will now assign an old shortcut to the action. Particularly
+    // picky apps might assert or misbehave.
     const QSet<QAction *> allActions = actions;
     nameToAction.clear();
     actions.clear();
@@ -535,28 +579,34 @@ QList<QStringList> KGlobalAccel::allActionsForComponent(const QStringList &actio
 }
 #endif
 
-//static
+// static
 #if KGLOBALACCEL_BUILD_DEPRECATED_SINCE(4, 2)
 QStringList KGlobalAccel::findActionNameSystemwide(const QKeySequence &seq)
 {
-    return self()->d->iface()->action(seq[0]);
+    return self()->d->iface()->actionList(seq);
 }
 #endif
 
+#if KGLOBALACCEL_BUILD_DEPRECATED_SINCE(5, 90)
 QList<KGlobalShortcutInfo> KGlobalAccel::getGlobalShortcutsByKey(const QKeySequence &seq)
 {
-    return self()->d->iface()->getGlobalShortcutsByKey(seq[0]);
+    return globalShortcutsByKey(seq);
+}
+#endif
+
+QList<KGlobalShortcutInfo> KGlobalAccel::globalShortcutsByKey(const QKeySequence &seq, MatchType type)
+{
+    return self()->d->iface()->globalShortcutsByKey(seq, type);
 }
 
 bool KGlobalAccel::isGlobalShortcutAvailable(const QKeySequence &seq, const QString &comp)
 {
-    return self()->d->iface()->isGlobalShortcutAvailable(seq[0], comp);
+    return self()->d->iface()->globalShortcutAvailable(seq, comp);
 }
 
-//static
+// static
 #if KGLOBALACCEL_BUILD_DEPRECATED_SINCE(4, 2)
-bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QStringList &actionIdentifier,
-        const QKeySequence &seq)
+bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QStringList &actionIdentifier, const QKeySequence &seq)
 {
     if (actionIdentifier.size() < 4) {
         return false;
@@ -565,9 +615,7 @@ bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QStringL
     QString message = tr("The '%1' key combination has already been allocated "
                          "to the global action \"%2\" in %3.\n"
                          "Do you want to reassign it from that action to the current one?")
-                      .arg(seq.toString(),
-                           actionIdentifier.at(KGlobalAccel::ActionFriendly),
-                           actionIdentifier.at(KGlobalAccel::ComponentFriendly));
+                          .arg(seq.toString(), actionIdentifier.at(KGlobalAccel::ActionFriendly), actionIdentifier.at(KGlobalAccel::ComponentFriendly));
 
     QMessageBox box(parent);
     box.setWindowTitle(title);
@@ -579,11 +627,8 @@ bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QStringL
 }
 #endif
 
-//static
-bool KGlobalAccel::promptStealShortcutSystemwide(
-    QWidget *parent,
-    const QList<KGlobalShortcutInfo> &shortcuts,
-    const QKeySequence &seq)
+// static
+bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QList<KGlobalShortcutInfo> &shortcuts, const QKeySequence &seq)
 {
     if (shortcuts.isEmpty()) {
         // Usage error. Just say no
@@ -594,16 +639,13 @@ bool KGlobalAccel::promptStealShortcutSystemwide(
 
     QString message;
     if (shortcuts.size() == 1) {
-        message = tr("The '%1' key combination is registered by application %2 for action %3:")
-                  .arg(seq.toString(), component, shortcuts[0].friendlyName());
+        message = tr("The '%1' key combination is registered by application %2 for action %3.").arg(seq.toString(), component, shortcuts[0].friendlyName());
     } else {
         QString actionList;
         for (const KGlobalShortcutInfo &info : shortcuts) {
-            actionList += tr("In context '%1' for action '%2'\n")
-                          .arg(info.contextFriendlyName(), info.friendlyName());
+            actionList += tr("In context '%1' for action '%2'\n").arg(info.contextFriendlyName(), info.friendlyName());
         }
-        message = tr("The '%1' key combination is registered by application %2.\n%3")
-                  .arg(seq.toString(), component, actionList);
+        message = tr("The '%1' key combination is registered by application %2.\n%3").arg(seq.toString(), component, actionList);
     }
 
     QString title = tr("Conflict With Registered Global Shortcut");
@@ -617,22 +659,23 @@ bool KGlobalAccel::promptStealShortcutSystemwide(
     return box.exec() == QMessageBox::Ok;
 }
 
-//static
+// static
 void KGlobalAccel::stealShortcutSystemwide(const QKeySequence &seq)
 {
-    //get the shortcut, remove seq, and set the new shortcut
-    const QStringList actionId = self()->d->iface()->action(seq[0]);
+    // get the shortcut, remove seq, and set the new shortcut
+    const QStringList actionId = self()->d->iface()->actionList(seq);
     if (actionId.size() < 4) { // not a global shortcut
         return;
     }
-    QList<int> sc = self()->d->iface()->shortcut(actionId);
+    QList<QKeySequence> sc = self()->d->iface()->shortcutKeys(actionId);
 
-    for (int i = 0; i < sc.count(); i++)
-        if (sc[i] == seq[0]) {
-            sc[i] = 0;
+    for (int i = 0; i < sc.count(); i++) {
+        if (sc[i] == seq) {
+            sc[i] = QKeySequence();
         }
+    }
 
-    self()->d->iface()->setForeignShortcut(actionId, sc);
+    self()->d->iface()->setForeignShortcutKeys(actionId, sc);
 }
 
 bool checkGarbageKeycode(const QList<QKeySequence> &shortcut)
@@ -641,7 +684,11 @@ bool checkGarbageKeycode(const QList<QKeySequence> &shortcut)
     // at the moment (~mid 2008) Multimedia PlayPause is one of those keys.
     for (const QKeySequence &sequence : shortcut) {
         for (int i = 0; i < 4; i++) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+            if (sequence[i].toCombined() == -1) {
+#else
             if (sequence[i] == -1) {
+#endif
                 qWarning() << "Encountered garbage keycode (keycode = -1) in input, not doing anything.";
                 return true;
             }
@@ -690,7 +737,7 @@ QList<QKeySequence> KGlobalAccel::shortcut(const QAction *action) const
     return d->actionShortcuts.value(action);
 }
 
-QList<QKeySequence> KGlobalAccel::globalShortcut(const QString& componentName, const QString& actionId) const
+QList<QKeySequence> KGlobalAccel::globalShortcut(const QString &componentName, const QString &actionId) const
 {
     // see also d->updateGlobalShortcut(action, KGlobalAccelPrivate::ActiveShortcut, KGlobalAccel::Autoloading);
 
@@ -698,8 +745,7 @@ QList<QKeySequence> KGlobalAccel::globalShortcut(const QString& componentName, c
     // action->setProperty("componentName", "kwin");
     // action->setObjectName("Kill Window");
 
-    const QList<int> result = self()->d->iface()->shortcut({ componentName, actionId, QString(), QString() });
-    const QList<QKeySequence> scResult(d->shortcutFromIntList(result));
+    const QList<QKeySequence> scResult = self()->d->iface()->shortcutKeys({componentName, actionId, QString(), QString()});
     return scResult;
 }
 
@@ -731,7 +777,7 @@ bool KGlobalAccel::setGlobalShortcut(QAction *action, const QKeySequence &shortc
     return KGlobalAccel::setGlobalShortcut(action, QList<QKeySequence>() << shortcut);
 }
 
-bool KGlobalAccelPrivate::setShortcutWithDefault(QAction* action, const QList<QKeySequence>& shortcut, KGlobalAccel::GlobalShortcutLoading loadFlag)
+bool KGlobalAccelPrivate::setShortcutWithDefault(QAction *action, const QList<QKeySequence> &shortcut, KGlobalAccel::GlobalShortcutLoading loadFlag)
 {
     if (checkGarbageKeycode(shortcut)) {
         return false;
@@ -747,5 +793,22 @@ bool KGlobalAccelPrivate::setShortcutWithDefault(QAction* action, const QList<QK
     return true;
 }
 
-#include "moc_kglobalaccel.cpp"
+QDBusArgument &operator<<(QDBusArgument &argument, const KGlobalAccel::MatchType &type)
+{
+    argument.beginStructure();
+    argument << static_cast<int>(type);
+    argument.endStructure();
+    return argument;
+}
 
+const QDBusArgument &operator>>(const QDBusArgument &argument, KGlobalAccel::MatchType &type)
+{
+    argument.beginStructure();
+    int arg;
+    argument >> arg;
+    type = static_cast<KGlobalAccel::MatchType>(arg);
+    argument.endStructure();
+    return argument;
+}
+
+#include "moc_kglobalaccel.cpp"
