@@ -65,7 +65,9 @@ public:
     void cleanup();
 
     // private slot implementations
+    QAction *findAction(const QString &, const QString &);
     void invokeAction(const QString &, const QString &, qlonglong);
+    void invokeDeactivate(const QString &, const QString &);
     void shortcutGotChanged(const QStringList &, const QList<int> &);
     void shortcutsChanged(const QStringList &, const QList<QKeySequence> &);
     void serviceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner);
@@ -100,6 +102,7 @@ public:
 
 private:
     org::kde::KGlobalAccel *m_iface = nullptr;
+    QPointer<QAction> m_lastActivatedAction;
     QDBusServiceWatcher *m_watcher;
 };
 
