@@ -10,7 +10,6 @@
 #include "globalshortcutsregistry.h"
 #include "kglobalaccel_interface.h"
 #include "logging_p.h"
-#include "sequencehelpers_p.h"
 #include <config-kglobalaccel.h>
 
 #include <QKeySequence>
@@ -30,9 +29,9 @@ static QList<QKeySequence> keysFromString(const QString &str)
     if (str == QLatin1String("none")) {
         return ret;
     }
-    const QStringList strList = str.split('\t');
+    const QStringList strList = str.split(QLatin1Char('\t'));
     for (const QString &s : strList) {
-        QKeySequence key = QKeySequence(s);
+        QKeySequence key = QKeySequence::fromString(s, QKeySequence::PortableText);
         if (!key.isEmpty()) { // sanity check just in case
             ret.append(key);
         }
@@ -47,40 +46,28 @@ static QString stringFromKeys(const QList<QKeySequence> &keys)
     }
     QString ret;
     for (const QKeySequence &key : keys) {
-        ret.append(key.toString());
-        ret.append('\t');
+        ret.append(key.toString(QKeySequence::PortableText));
+        ret.append(QLatin1Char('\t'));
     }
     ret.chop(1);
     return ret;
 }
 
-namespace KdeDGlobalAccel
-{
-Component::Component(const QString &uniqueName, const QString &friendlyName, GlobalShortcutsRegistry *registry)
+Component::Component(const QString &uniqueName, const QString &friendlyName)
     : _uniqueName(uniqueName)
     , _friendlyName(friendlyName)
-    , _registry(registry)
+    , _registry(GlobalShortcutsRegistry::self())
 {
     // Make sure we do no get uniquenames still containing the context
-    Q_ASSERT(uniqueName.indexOf("|") == -1);
+    Q_ASSERT(uniqueName.indexOf(QLatin1Char('|')) == -1);
 
-    // Register ourselves with the registry
-    if (_registry) {
-        _registry->addComponent(this);
-    }
-
-    QString DEFAULT = QStringLiteral("default");
+    const QString DEFAULT(QStringLiteral("default"));
     createGlobalShortcutContext(DEFAULT, QStringLiteral("Default Context"));
     _current = _contexts.value(DEFAULT);
 }
 
 Component::~Component()
 {
-    // Remove ourselves from the registry
-    if (_registry) {
-        _registry->takeComponent(this);
-    }
-
     // We delete all shortcuts from all contexts
     qDeleteAll(_contexts);
 }
@@ -88,7 +75,7 @@ Component::~Component()
 bool Component::activateGlobalShortcutContext(const QString &uniqueName)
 {
     if (!_contexts.value(uniqueName)) {
-        createGlobalShortcutContext(uniqueName, "TODO4");
+        createGlobalShortcutContext(uniqueName, QStringLiteral("TODO4"));
         return false;
     }
 
@@ -158,15 +145,20 @@ GlobalShortcutContext *Component::currentContext()
 
 QDBusObjectPath Component::dbusPath() const
 {
+    auto isNonAscii = [](QChar ch) {
+        const char c = ch.unicode();
+        const bool isAscii = c == '_' //
+            || (c >= 'A' && c <= 'Z') //
+            || (c >= 'a' && c <= 'z') //
+            || (c >= '0' && c <= '9');
+        return !isAscii;
+    };
+
     QString dbusPath = _uniqueName;
-    // Clean up for dbus usage: any non-alphanumeric char should be turned into '_'
-    const int len = dbusPath.length();
-    for (int i = 0; i < len; ++i) {
-        if (!dbusPath[i].isLetterOrNumber() || dbusPath[i].unicode() >= 0x7F) {
-            // DBus path can only contain ASCII characters
-            dbusPath[i] = QLatin1Char('_');
-        }
-    }
+    // DBus path can only contain ASCII characters, any non-alphanumeric char should
+    // be turned into '_'
+    std::replace_if(dbusPath.begin(), dbusPath.end(), isNonAscii, QLatin1Char('_'));
+
     // QDBusObjectPath could be a little bit easier to handle :-)
     return QDBusObjectPath(_registry->dbusPath().path() + QLatin1String("component/") + dbusPath);
 }
@@ -313,7 +305,7 @@ Component::registerShortcut(const QString &uniqueName, const QString &friendlyNa
     QList<QKeySequence> newKeys = keys;
     for (const QKeySequence &key : keys) {
         if (!key.isEmpty()) {
-            if (GlobalShortcutsRegistry::self()->getShortcutByKey(key)) {
+            if (_registry->getShortcutByKey(key)) {
                 // The shortcut is already used. The config file is
                 // broken. Ignore the request.
                 newKeys.removeAll(key);
@@ -419,4 +411,4 @@ void Component::writeSettings(KConfigGroup &configGroup) const
     }
 }
 
-} // namespace KdeDGlobalAccel
+#include "moc_component.cpp"
